# Main model for the effect of NLP

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

library(brms)
library(ggplot2)
library(soundgen)
source('ns1_helperFun.R')

df = read.csv('../data/ns1.csv', stringsAsFactors = TRUE)
df$nlp = factor(df$nlp, levels = c('none', 'subh', 'am', 'chaos'))
aggregate(respNorm ~ scale, df, range)

# observed
ggplot(df, aes(nlp, response)) +
  geom_boxplot() +
  facet_wrap(~scale, scales = 'free') +
  theme_bw()

# model
mod_file = '../mod/resp~nlp*scale.RDS'  # mod_file = '../mod/resp~nlp*scale_full.RDS'
if (file.exists(mod_file)) {
  mod = readRDS(mod_file)
} else {
  mod = brm(
    respNorm ~ nlp * scale + (nlp * scale | subject) + 
      (scale|sound) + (nlp * scale|sound_base), 
    data = df, family = 'zero_one_inflated_beta', 
    chains = 2, cores = 2, warmup = 500, iter = 4000,
    # inits = 0,  # start all chains from the same par values
    init_r = 0.1  # a better way to ensure convergence with a lot of random slopes - see https://discourse.mc-stan.org/t/model-with-many-correlated-varying-slopes-sampling-not-done/6189
  )  # ~19.4 h for 4000 iter, 30K observations
  # mod_sex = brm(
  #   respNorm ~ nlp * scale * speakerSex + (nlp * scale | subject) + 
  #     (scale|sound) + (nlp * scale|sound_base), 
  #   data = df, family = 'zero_one_inflated_beta', 
  #   chains = 2, cores = 2, warmup = 500, iter = 4000
  # ) 
  # loo(mod, mod_sex)  # tested on a subset of data (pilot 1)
  # Model comparisons:
  #   elpd_diff se_diff
  # mod      0.0       0.0   
  # mod_sex -2.1       7.1   
  saveRDS(mod, mod_file)
}
# summary(mod)
# plot(mod)
# pp_check(mod)
# conditional_effects(mod)

# extract fitted values per nlp
newdata = expand.grid(
  nlp = levels(df$nlp),
  scale = levels(df$scale),
  stringsAsFactors = FALSE
)
fit = fitted(mod, newdata = newdata, re_formula = NA, robust = TRUE)
colnames(fit) = c('fit', 'se', 'lwr', 'upr')
plot_fit = cbind(newdata, fit)
plot_fit$nlp = factor(plot_fit$nlp, levels = rev(levels(df$nlp)))
plot_fit$scale = factor(plot_fit$scale, levels = c('pitch', 'timbre', 'roughness', 'height', 'formidability', 'aggression'))
mean(plot_fit$upr - plot_fit$lwr)  # average CI per condition is ~9.6% wide
plot_fit = deNorm(plot_fit)
plot_fit$nlpFmt = factor(as.character(names_nlps[as.character(plot_fit$nlp)]), levels = rev(as.character(names_nlps)))
plot_fit$scaleFmt = factor(as.character(names_scales[as.character(plot_fit$scale)]), levels = as.character(names_scales))

# extract fitted values per nlp and sound
newdata_sound = expand.grid(
  sound = levels(df$sound),
  scale = levels(df$scale),
  stringsAsFactors = FALSE
)
newdata_sound[, c('sound_base', 'nlp')] = df[match(newdata_sound$sound, df$sound), c('sound_base', 'nlp')]
fit_sound = fitted(mod, newdata = newdata_sound, 
                   re_formula = '~(scale|sound)+(nlp * scale|sound_base)', robust = TRUE)
colnames(fit_sound) = c('fit', 'se', 'lwr', 'upr')
plot_fit_sound = cbind(newdata_sound, fit_sound)
plot_fit_sound$nlp = factor(plot_fit_sound$nlp, levels = rev(levels(df$nlp)))
mean(plot_fit_sound$upr - plot_fit_sound$lwr)  # average CI per sound is ~12.5% wide
# save on a 01 scale, before deNorm'ing
# write.csv(plot_fit_sound, '../data/fit_sound.csv', row.names = FALSE)
plot_fit_sound = deNorm(plot_fit_sound)
plot_fit_sound$nlpFmt = factor(as.character(names_nlps[as.character(plot_fit_sound$nlp)]), levels = rev(as.character(names_nlps)))
plot_fit_sound$scaleFmt = factor(as.character(names_scales[as.character(plot_fit_sound$scale)]), levels = as.character(names_scales))

ggplot(plot_fit, aes(x = nlpFmt, y = fit, ymin = lwr, ymax = upr)) +
  geom_violin(data = plot_fit_sound, aes(x = nlpFmt, y = fit, fill = pitch), color = NA, fill = 'gray50', alpha = .25, inherit.aes = FALSE, position = position_dodge(.7)) +
  geom_pointrange(position = position_dodge(.7)) +
  facet_wrap(~scaleFmt, scales = 'free_x') +
  xlab('') +
  ylab('Rating (fitted values)') +
  coord_flip() +
  theme_bw() +
  theme(panel.grid = element_blank())
# ggsave('../pix/nlp_fitted.png', width = 20, height = 12, units = 'cm', dpi = 600)


# contrasts vs "none"
fit1 = fitted(mod, newdata = newdata, re_formula = NA, summary = FALSE)
contr = newdata[newdata$nlp != 'none', ]
for (i in 1:nrow(contr)) {
  idx = which(newdata$scale == contr$scale[i])
  idx_i = idx[which(newdata$nlp[idx] == contr$nlp[i])]
  idx_none = idx[which(newdata$nlp[idx] == 'none')]
  d = fit1[, idx_i] - fit1[, idx_none]
  contr[i, c('fit', 'lwr', 'upr')] = quantile(d, probs = c(.5, .025, .975))
}
mean(contr$upr - contr$lwr)  # 4.3% before renorming
contr = deNorm(contr, centered = TRUE)

contr$nlpFmt = factor(as.character(names_nlps[as.character(contr$nlp)]), levels = rev(as.character(names_nlps)))
contr$scaleFmt = factor(as.character(names_scales[as.character(contr$scale)]), levels = as.character(names_scales))

# contrasts for each prototype
fit1_sound = fitted(mod, newdata = newdata_sound, 
                    re_formula = '~(scale|sound)+(nlp * scale|sound_base)', summary = FALSE)
contr_sound = newdata_sound[newdata_sound$nlp != 'none', ]
for (i in 1:nrow(contr_sound)) {
  idx_i = which(newdata_sound$sound_base == contr_sound$sound_base[i] &
                  newdata_sound$nlp == contr_sound$nlp[i] &
                  newdata_sound$scale == contr_sound$scale[i])
  idx_none = which(newdata_sound$sound_base == contr_sound$sound_base[i] & 
                     newdata_sound$nlp == 'none' &
                     newdata_sound$scale == contr_sound$scale[i])
  d = fit1_sound[, idx_i] - fit1_sound[, idx_none]
  contr_sound[i, c('fit', 'lwr', 'upr')] = quantile(d, probs = c(.5, .025, .975))
}
contr_sound = deNorm(contr_sound, centered = TRUE)
mean(contr_sound$upr - contr_sound$lwr)  # average CI per sound_base is ~13% wide
contr_sound$nlpFmt = factor(as.character(names_nlps[as.character(contr_sound$nlp)]), levels = rev(as.character(names_nlps)))
contr_sound$scaleFmt = factor(as.character(names_scales[as.character(contr_sound$scale)]), levels = as.character(names_scales))
# write.csv(contr_sound, '../data/contr_sound.csv', row.names = FALSE)

ggplot(contr, aes(x = nlpFmt, y = fit, ymin = lwr, ymax = upr)) +
  geom_violin(data = contr_sound, aes(x = nlpFmt, y = fit), color = NA, fill = 'gray50', alpha = .25, inherit.aes = FALSE) +
  geom_pointrange(pch = 16, color = 'black', fill = 'black') +
  geom_hline(yintercept = 0, linetype = 3) +
  facet_wrap(~scaleFmt, nrow = 2, scales = 'free_x') +
  xlab('') +
  ylab('Effect of adding NLPs') +
  coord_flip() +
  theme_bw() +
  theme(panel.grid = element_blank())
# ggsave('../pix/contrasts_perNLP.png', width = 16, height = 10, units = 'cm', dpi = 600)

contr$ci = soundgen:::reportCI(contr[, c('fit', 'lwr', 'upr')], 1)
contr[, c('nlp', 'scale', 'ci')]
