# The effect of NLP on size in pairwise comparisons

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

library(brms)
library(ggplot2)

## nlps
df = read.csv('../data/ns2.csv')
length(unique(df$subject))  # 192 participants
nrow(df) / length(unique(c(df$sound1, df$sound2)))  # 29 ratings/stimulus
nrow(df) / length(unique(df$sound_base)) / 6  # 20 ratings/pair
length(unique(df$sound_base))  # 82
length(unique(df$sound1))  # 328

# Pretty names for plotting
names_nlps = list(
  'none' = 'No NLPs',
  'subh' = 'Subharmonics',
  'am' = 'Amplitude\nmodulation',
  'chaos' = 'Chaos'
)
nlp_order = data.frame(nlp = rev(names(names_nlps)), num = 1:4)

df$pair = NA
df$size = NA
df$nlp1 = as.character(df$nlp1)
df$nlp2 = as.character(df$nlp2)
for (i in 1:nrow(df)) {
  df$resp_num[i] = switch(as.character(df$response[i]),
                           buttonLeft = 1,
                           buttonMid = 2,
                           buttonRight = 3)
  vv = df[i, c('nlp1', 'nlp2')]
  if (vv[1] == vv[2]) {
    # catch trial
    df$pair[i] = 'catch'
    df$size[i] = df$resp_num[i]
    df$pos[i] = 'left'
  } else {
    # two different nlps
    nlps_i = vv[order(na.omit(match(nlp_order$nlp, vv)))]
    df$pair[i] = paste0(nlps_i, collapse = '_')  # paste0(sort(vv), collapse = '_')
    if (vv[1] == nlps_i[1]) {
      df$pos[i] = 'left'  # ie reversed
      df$size[i] = 4 - df$resp_num[i]
    } else {
      df$pos[i] = 'right'
      df$size[i] = df$resp_num[i]
    }
  }
}
# "df$size" shows preference for the first in "df$pair"
head(df)
table(df$size, df$pair)

av = aggregate(size ~ pair, df, mean)
av = av[order(av$size), ]
plot(av$size, type = 'n')
text(1:nrow(av), av$size, labels = av$pair)

# model
# aggregate(size ~ pair, df, mean)
mod_file = '../mod/nlpPairs.RDS'
if (file.exists(mod_file)) {
  mod = readRDS(mod_file)
} else {
  mod = brm(size ~ pair + (pair|subject) + (pair|sound_base), data = df, family = 'cumulative', iter = 4000, warmup = 500, chains = 2, cores = 2, init_r = 0.1)  # a way to ensure convergence with a lot of random slopes - see https://discourse.mc-stan.org/t/model-with-many-correlated-varying-slopes-sampling-not-done/6189
  saveRDS(mod, mod_file)
s}
# summary(mod)
# pp_check(mod)
# conditional_effects(mod, categorical = TRUE)

newdata = data.frame(pair = unique(df$pair))
f = fitted(mod, newdata = newdata, re_formula = NA, robust = TRUE)
# f = f[,, 3]
colnames(f) = c('fit', 'se', 'lwr', 'upr')

df_plot = cbind(newdata, f)

df_plot = df_plot[order(df_plot$fit), ]
df_plot$pair = factor(df_plot$pair, levels = df_plot$pair)

ggplot(df_plot, aes(x = pair, y = fit, ymin = lwr, ymax = upr)) +
  geom_errorbar() +
  geom_point()

# contrasts
f1 = fitted(mod, newdata = newdata, robust = TRUE, re_formula = NA, summary = FALSE)
# dim(f1)
# newdata
apply(f1[, , 3], 2, function(x) quantile(x, probs = .5))

df_contr = newdata
df_contr[, c('fit', 'lwr', 'upr')] = NA
for (i in 1:nrow(df_contr)) {
  idx_cond = which(newdata$pair == df_contr$pair[i])
  contrast = (f1[, idx_cond, 3] - f1[, idx_cond, 1])
  contrast_summary = quantile(contrast, probs = c(.5, .025, .975)) * 100
  df_contr[i, c('fit', 'lwr', 'upr')] = contrast_summary
}

# ggplot(df_contr, aes(x = cond, y = fit, ymin = lwr, ymax = upr, group = animal, col = animal, shape = animal, fill = animal)) +
#   geom_col(position = position_dodge(.7), width = .7) +
#   geom_errorbar(position = position_dodge(.7), width = .1, col = 'black') +
#   geom_hline(yintercept = 0, linetype = 2) +
#   theme_bw()

df_contr = df_contr[order(df_contr$fit), ]
df_contr$pair = factor(df_contr$pair, levels = df_contr$pair[order(df_contr$fit)])
idx_catch = which(df_contr$pair == 'catch')

rope = as.numeric(df_contr[idx_catch, c('lwr', 'upr')])
df_contr$clearRope = (df_contr$lwr > rope[2] | df_contr$upr < rope[1])
df_rope = data.frame(xmin = -Inf, xmax = Inf, ymin = rope[1], ymax = rope[2])

for (i in 1:nrow(df_contr)) {
  df_contr$pairFmt[i] = switch(as.character(df_contr$pair[i]),
                               'am_subh' = 'AM vs Subharmonics',
                               'am_none' = 'AM vs None',
                               'subh_none' = 'Subharmonics vs None',
                               'chaos_subh' = 'Chaos vs Subharmonics',
                               'chaos_am' = 'Chaos vs AM',
                               'chaos_none' = 'Chaos vs None',
                               'catch')
}
df_contr$pairFmt = factor(df_contr$pairFmt, levels = df_contr$pairFmt[order(df_contr$fit)])

# contrasts per sound_base
newdata_sb = expand.grid(pair = unique(df$pair),
                         sound_base = unique(df$sound_base))
fit_sb = fitted(mod, newdata = newdata_sb, robust = TRUE, re_formula = '~(pair|sound_base)', summary = FALSE)
# dim(fit_sb)
contr_sb = newdata_sb
contr_sb[, c('fit', 'lwr', 'upr')] = NA
for (i in 1:nrow(contr_sb)) {
  contrast = (fit_sb[, i, 3] - fit_sb[, i, 1])
  contrast_summary = quantile(contrast, probs = c(.5, .025, .975)) * 100
  contr_sb[i, c('fit', 'lwr', 'upr')] = contrast_summary
}
# head(contr_sb)
for (i in 1:nrow(contr_sb)) {
  contr_sb$pairFmt[i] = switch(as.character(contr_sb$pair[i]),
                               'am_subh' = 'AM vs Subharmonics',
                               'am_none' = 'AM vs None',
                               'subh_none' = 'Subharmonics vs None',
                               'chaos_subh' = 'Chaos vs Subharmonics',
                               'chaos_am' = 'Chaos vs AM',
                               'chaos_none' = 'Chaos vs None',
                               'catch')
}
contr_sb$pairFmt = factor(contr_sb$pairFmt, levels = levels(df_contr$pairFmt))
# write.csv(contr_sb, '../data/contrastsPerSound.csv', row.names = FALSE)

ggplot(df_contr[-idx_catch, ], aes(x = pairFmt, y = fit, ymin = lwr, ymax = upr, alpha = clearRope)) +
   geom_violin(data = contr_sb[-which(contr_sb$pair == 'catch'), ], aes(x = pairFmt, y = fit), inherit.aes = FALSE, color = NA, fill = 'gray50', alpha = .5) +
  geom_rect(data = df_rope, aes(xmin = xmin, xmax = xmax, ymin = ymin, ymax = ymax), color = NA, fill = 'gray50', alpha = .15, inherit.aes = FALSE) +
  geom_pointrange(pch = 16, color = 'black', fill = 'black') +
  geom_hline(yintercept = 0, linetype = 2) +
  xlab('') +
  ylab('p(1st larger) - p(2nd larger), %') +
  scale_alpha_discrete(range = c(.3, 1), guide = FALSE) +
  coord_flip() +
  theme_bw() +
  theme(panel.grid = element_blank(),
        legend.title = element_blank(),
        axis.title.x = element_text(size = 9))

# ggsave(filename = '../pix/nlpPairs.png', width = 8.5, height = 5, units = 'cm', dpi = 600)

df_contr$ci = soundgen:::reportCI(df_contr[, c('fit', 'lwr', 'upr')], 1)
df_contr[, c('pair', 'ci')]

# variation in the effect of chaos per prototype
sum(contr_sb$fit[contr_sb$pair == 'chaos_none'] < df_rope$ymax)
sum(contr_sb$fit[contr_sb$pair == 'chaos_none'] < df_rope$ymin)

